#include <HT32F125x.h>                    // 

#include "SED1335_lowlevel.h"
#include "8X15FAT.H"
#include "font_pearl8x8.h"

unsigned char LCD_font=0;

struct fonts {
  uint8_t v;
  const uint8_t *font;
};

const static struct fonts font[]={
  {8,font_pearl8x8},
  {15,Font8x15Fat},
};


/* Low level functions */
void LCD_WriteCmd(unsigned char cmd)
{
	LCD_A0=1;	// Rise A0 (Cmd)
	LCD_Write(cmd);
	LCD_A0=0;	// Lower A0 (Data)
}

void LCD_WriteStrCmd(unsigned char *cmd,unsigned char len)
{
	LCD_WriteCmd(*cmd++);
	while(--len) {
		LCD_Write(*cmd++);
	}
}


unsigned char LCD_ReadData()
{

unsigned char dat;

	LCD_A0=1;	// Rise A0 (Read Data)
	dat=LCD_Read();
	LCD_A0=0;	// Lower A0
	return(dat);
}


void LCD_init ()
{
extern void Delay (unsigned long);
const static uint8_t init_string1[]={
0x40,				// INIT
0x30,	                 // 32 char CGRAM, 8 pixel chars, no top-screen corr, LCD, normal
0x87,0x7,		// 8x8 characters
39,59,			// 320 pixels, 8 pixel characters => 40 characters per line (43)
239,				// 240 lines (30 txt lines)
40,0				// virtual screen = display screen
};

const static uint8_t init_string2[]={
0x44,				// SCROLL
0, 				// SAD1L: start address screen 1 (0x0000)
0, 				// SAD1H: (for text screen)
239, 				// SL1  : number of scrolling lines screen 1 (239 dec.)
0xB0, 			// SAD2L: start address screen 2 (1200=0x4b0)
0x04, 			// SAD2H: (for the graphics screen)
239, 				// SL2  : number of scrolling lines screen 2
0, 				// SAD3L:
0, 				// SAD3H:
0, 				// SAD4L:
0	 				// SAD4H:
};

const static uint8_t init_string3[]={
0x5D,				// CSRFORM	 
0x04,				// CRX: set the horizontal size of the cursor (4 pixel)
0x86				// CRY: set the vertical size of the cursor (6 pixel) CM : set the cursor display mode to block cursor
};

		
const static uint8_t init_string4[]={
0x5A,				// HDOT_SCR (0x5a), Sets horizontal scroll position (scroll rate).
0					// 1 pixel scroll
};

const static uint8_t init_string5[]={
0x5B,				// OVLAY, Selects layered screen composition and screen text/graphics mode.
0x01      	//MX0, MX1: set the text/graphics overlay mode to EXOR
//		Bit 2-3 : DM1, DM2: set the display mode of screen block 1 and 3
//		Bit 4   : OV: set two or three layer compsition in gfx-mode
//		Bit 5-7 :  0: not used (set to 0)
};

const static uint8_t init_string6[]={
0x59,				// send command DISP_ON (0x59)
0x16                  	// cursor flash rate = 2 Hz, SAD1/2 no flashing, SAD3 off
};

	LCD_setupIO();
	Delay ( 1000 );						// wait for LCD reset
	LCD_WriteStrCmd ((unsigned char *) init_string1,sizeof(init_string1) );
	LCD_WriteStrCmd ((unsigned char *) init_string2,sizeof(init_string2) );
	LCD_WriteStrCmd ((unsigned char *) init_string3,sizeof(init_string3) );
	LCD_WriteCmd ( 0x4C );		// CSRDIR_RIGHT, sets the direction of automatic cursor increment to right.
	LCD_WriteStrCmd ((unsigned char *) init_string4,sizeof(init_string4) );
	LCD_WriteStrCmd ((unsigned char *) init_string5,sizeof(init_string5) );
	LCD_WriteStrCmd ((unsigned char *) init_string6,sizeof(init_string6) );
}


/* High level functions */
void LCD_cursor ( unsigned int address )
{
	LCD_WriteCmd(0x46);									// CSRW, Sets cursor address.
	LCD_Write(address&0xFF);							// LO byte
	LCD_Write((((unsigned int)address)>>8)&0xFF);		// HI byte
}

void LCD_fill(unsigned int address,unsigned char width,unsigned char height,unsigned char pattern)
{
unsigned int w;

   while(height--){
     LCD_cursor(address);
     LCD_WriteCmd(0x42);									// MWRITE
     w=width;
     while(w--){
       LCD_Write(pattern);							// fill with pattern
	}
     address+=40;
   }
}


void LCD_printat (unsigned char row, unsigned int col, char *ptr)
{
	LCD_cursor ((row<<5)+(row<<3)+col);	// 40*y+col						// set cursor address
	LCD_WriteCmd(0x42);									// MWRITE
 	while (*ptr)
		LCD_Write (*ptr++);
}

/* Plot image data, 8 pixels per byte, MSB left */

void LCD_plot (unsigned int x, unsigned char y)
{
unsigned char strip,bt,dat;
unsigned int addr;

	strip=((unsigned int)x)>>3;					// strip = x/8
	bt=(unsigned char)x & 0x07;						// bit = remainder
	addr=(y<<5)+(y<<3);              		        // 40*y
	addr+=1200+strip;
	LCD_cursor(addr);								// address strip in gfx screen
	LCD_WriteCmd(0x43);								// MREAD
	dat=LCD_ReadData();								// get data at that address
	dat|=(((unsigned char)0x80)>>bt);				// set requested bit (MSB left)
	LCD_cursor(addr);								// address strip in gfx screen again
	LCD_WriteCmd(0x42);								// MWRITE
	LCD_Write(dat);
}


void LCD_icon(unsigned int address,unsigned char width,unsigned char height,uint8_t *imgdata)
{
unsigned int w;

   while(height--){
     LCD_cursor(address);
     LCD_WriteCmd(0x42);									// MWRITE
     w=width;
     while(w--)
	  LCD_Write(*(imgdata++));
     address+=40;
   }
}


// Bresenham Line Algorithm

void LCD_line(unsigned int x1,unsigned char y1,unsigned int x2,unsigned char y2)
{
unsigned char y;
int dy,incx,incy;
unsigned int x;
int dx,balance;

        if (x2 >= x1)
        {
                dx = x2 - x1;
                incx = 1;
        }
        else
        {
                dx = x1 - x2;
                incx = -1;
        }

        if (y2 >= y1)
        {
                dy = y2 - y1;
                incy = 1;
        }
        else
        {
                dy = y1 - y2;
                incy = -1;
        }

        x = x1;
        y = y1;

        if (dx >= dy)
        {
                dy <<= 1;
                balance = dy - dx;
                dx <<= 1;

                while (x != x2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                y += incy;
                                balance -= dx;
                        }
                        balance += dy;
                        x += incx;
                } LCD_plot(x, y);
        }
        else
        {
                dx <<= 1;
                balance = dx - dy;
                dy <<= 1;

                while (y != y2)
                {
                        LCD_plot(x, y);
                        if (balance >= 0)
                        {
                                x += incx;
                                balance -= dy;
                        }
                        balance += dx;
                        y += incy;
                } LCD_plot(x, y);
        }
}

void LCD_rectangle(unsigned int x1,unsigned char y1,unsigned int x2,unsigned char y2)
{
	LCD_line(x1,y1,x2,y1);
	LCD_line(x2,y1,x2,y2);
	LCD_line(x2,y2,x1,y2);
	LCD_line(x1,y2,x1,y1);
}

void LCD_printFat (unsigned char row, unsigned int col, char *ptr)
{
const uint8_t *cptr;
unsigned char v;
unsigned int addr;

 	while (*ptr){
	  addr=1200+(row<<8)+(row<<6)+col;	// 1200+320*y+col
	  v=font[LCD_font].v;
	  cptr=font[LCD_font].font+(*(ptr++)*v);
	  while(v--){
	    LCD_cursor(addr);	// set cursor address
	    LCD_WriteCmd(0x42);			// MWRITE
	    LCD_Write(*(cptr++));
	    addr+=40;
	  }
	  col++;
	}
}
