#include <HT32F125x.h>                    // 

#include "SED1335_lowlevel.h"

/* 72 MHz -> 14ns, while loop is ~3 instructions -> ~100ns
this does not take into account the AHB+APB internal clocking and estimates for flash 2 wait-states */
#define DILEI(x){volatile int mydelay;mydelay=(x);while(mydelay--);}

/* Low level functions */
void LCD_Write(unsigned char dat)
{
/* S1D13305: 0ns address setup, 120ns data setup, 120ns strobe width */
  
	HT_GPIOA->DOUTR = (dat<<8);			// write data
	if(dat&(1<<2))
		HT_GPIOB->DOUTR |= (1<<LCD_D2b);
	else
		HT_GPIOB->DOUTR &= ~(1<<LCD_D2b);
	HT_GPIOA->DIRCR |= 0xFB00;       // PA.8,9,11-15 defined as Outputs, put data in bus
	HT_GPIOB->DIRCR |= (1<<LCD_D2b);      // LCD_D2 defined as Output, put data in bus
	LCD_CS=0;		// Lower CS, ? ns
		// need 120 - 2x? ns delay; use AHB+APB (~100ns per cycle)
	LCD_WR=0;		// Lower WR, ? ns
		// need 120 - ? ns delay;
		DILEI(1);
	LCD_WR=1;		// Rise WR,  ? ns delay
	LCD_CS=1;		// Rise CS
}

unsigned char LCD_Read()
{
/* S1D13305: 0ns address setup, 50ns data delay, 120ns strobe width */

unsigned char dat;

	HT_GPIOA->DIRCR &= ~0xFB00;             // PA.8,9,11-15 defined as Inputs, remove data from bus
	HT_GPIOB->DIRCR &= ~(1<<LCD_D2b);       // LCD_D2 defined as Input, remove data from bus
	LCD_CS=0;		// Lower CS, ? ns
	LCD_RD=0;		// Lower RD, ? ns
		// need 50 - ?? ns delay; use AHB+APB (~100ns)
	dat=(unsigned char)(HT_GPIOA->DINR >> 8);	           // read data, ? ns
	if(HT_GPIOB->DINR & (1<<LCD_D2b))				// map LCD_D2 to D2, ? ns
		dat |= (1<<2);
	else
		dat &= ~(1<<2);
		// need 120 - 50 - 2x? ns delay; use AHB+APB (~100ns per cycle)
	LCD_RD=1;		// Rise RD,  ? ns delay
	LCD_CS=1;		// Rise CS
	return(dat);
}

void LCD_setupIO()
{
	HT_CKCU->APBCCR0 |= (1<<17)+(1<<16)+(1<<14);// 17=PBEN, 16=PAEN, 14=AFIOEN, enable APB clocks for GPIOA, GPIOB, and AFIO
	HT_AFIO->GPACFGR = 0x54000000;			// PA15,14,13 -> AF1 (lose SWD, no debugging)
	HT_GPIOA->INER &= ~0x0400;                      // PA.10 input circuit disabled
	HT_GPIOA->INER |= 0xFB00;                      // PA.8,9,11-15 input circuits enabled
	HT_GPIOB->INER |= (1<<LCD_D2b);                // LCD_D2 input circuit enabled
	HT_GPIOA->DIRCR |= 0xFF00;                      // PA.8-15 defined as Outputs
	HT_GPIOB->DOUTR |= (1<<LCD_CSb)+(1<<LCD_WRb)+(1<<LCD_RDb);
	HT_GPIOB->DIRCR |= 0xF800;                     // PB.11,12,13,14,15 defined as Outputs
	HT_GPIOA->DIRCR |= 0xFF00;                      // PA.8-15 defined as Outputs
}


